from eval_metrics import ClassifierEvalMultilabel
from utils import label_to_onehot
import numpy as np
import pickle as pkl
import matplotlib.pyplot as plt

if __name__ == '__main__':

    split = 'set1_res101'

    print('Evaluate multilabel classification according to predicted scores...')
    label_file = './input_file/{:s}/multilabel_val_label.pkl'.format(split)
    pred_file = './input_file/{:s}/multilabel_val_prediction.pkl'.format(split)
    with open(label_file, 'rb') as f: # list
        labels = pkl.load(f)
    labels = label_to_onehot(labels, 6)# [[0], [1, 2], [0]] -> [[1, 0, 0], [0, 1, 1], [1, 0, 0]]
    with open(pred_file, 'rb') as f: # numpy
        predicts = pkl.load(f)
    ap_dict = ClassifierEvalMultilabel.compute_ap(labels, predicts)
    prec_dict = ClassifierEvalMultilabel.compute_p_at_r(labels, predicts, recall_thresh=0.995)
    ClassifierEvalMultilabel.draw_pr_curve(labels, predicts, output_dir='111')
    min_score, res_dict, fn_index_list, fp_index_list = ClassifierEvalMultilabel.compute_fnr_and_fpr(labels, predicts, ok_ind=0, fnr_thresh=0.005)
    print('AP res is: ', ap_dict)
    print('P at R res is: ', prec_dict)
    print('Loubao and wubao res is: ', res_dict)
    print('Min ng score is: ', min_score)


    print('Evaluate multilabel classification according to labels...')
    for i, pred in enumerate(predicts):
        for j, item in enumerate(pred[1:6]):
            predicts[i, j+1] = 1 if item >= min_score else 0
        if sum(pred[1:6]) >= 1:
            predicts[i, 0] = 0
        else:
            predicts[i, 0] = 1
            
    # cnt = 0
    # for i, pred in enumerate(predicts):
    #     if pred[0] == 1 and sum(pred) != 1:
    #         cnt += 1
    # print(cnt)

    ap_dict = ClassifierEvalMultilabel.compute_ap(labels, predicts)
    prec_dict = ClassifierEvalMultilabel.compute_p_at_r(labels, predicts, recall_thresh=0.995)
    ClassifierEvalMultilabel.draw_pr_curve(labels, predicts, output_dir='111')
    min_score, res_dict, fn_index_list, fp_index_list = ClassifierEvalMultilabel.compute_fnr_and_fpr(labels, predicts, ok_ind=0, fnr_thresh=0.005)
    print('AP res is: ', ap_dict)
    print('P at R res is: ', prec_dict)
    print('Loubao and wubao res is: ', res_dict)
    print('Min ng score is: ', min_score) # 此处min_score没有意义

    # 将从prediction score得到的label写入文件
    with open('./input_file/{:s}/multilabel_val_predict_label.pkl'.format(split), 'wb') as f:
        pkl.dump(predicts, f)
    

    # # 获取一张图片中有多个缺陷的图片
    # with open('./input_file/multilabel_val_path.pkl', 'rb') as f:
    #     img_paths = pkl.load(f)
    # for i, label in enumerate(labels):
    #     if sum(label) > 1:
    #         print(label)
    #         print(img_paths[i])


import uuid
import hashlib
import os


def create_uuid():
    """ create a uuid (universally unique ID) """
    md5_hash = hashlib.md5(uuid.uuid1().bytes)
    return md5_hash.hexdigest()


def get_hash_code(file):
    """ get the md5 hash code of a given file """
    assert os.path.exists(file)
    md5_hash = hashlib.md5()
    with open(file, "rb") as fid:
        md5_hash.update(fid.read())
        digest = md5_hash.hexdigest()
    return digest


def is_black_distribution(distribution):
    """ Check if the distribution is implying a black instance (used in data.update()) """
    return all([x < 0 for x in distribution])




#
# Class for evaluation (classification/detection/segmentation)
#
from os import isatty
from .data_manager import DataManager
from .anno_manager import AnnotationManager
from .utils import ClassifierEvalMultilabel
from .utils import get_TFPN
import numpy as np
import pandas as pd
from sklearn.metrics import confusion_matrix as compute_confusion_matrix

class EvaluatorBase(object):
    def __init__(self, classname_list=None, data_gt=None, data_pred=None):
        """
        :classname_list: list of classes that we are interested in
        """
        self._classname_list = classname_list
        self._data_gt = data_gt
        self._data_pred = data_pred
        # self.am = None

    def _load_data(self, data):
        if isinstance(data, str):
            return DataManager.load(data)
        elif isinstance(data, DataManager):
            return data
        else:
            raise RuntimeError("Unknown data format: {}".format(type(data)))

    def load_gt(self, data):
        self._data_gt = self._load_data(data)

    def load_pred(self, data):
        self._data_pred = self._load_data(data)

    def set_classnames(self, classname_list):
        if isinstance(classname_list, (tuple, list)):
            self._classname_list = classname_list
        elif isinstance(classname_list, str):
            self._classname_list = [classname_list]
        else:
            raise RuntimeError("Unknown data format: {}".format(type(classname_list)))

    def _check_data(self):
        assert self._data_gt and self._data_pred, "Please load both gt and pred before evaluation."
        assert self._data_gt == self._data_pred, "Make sure the gt data and pred data match"


    # aliases
    load_anno = load_gt
    load_annotation = load_gt
    load_groundtruth = load_gt
    load_prediction = load_pred
    set_class = set_classnames
    set_classname = set_classnames
    set_classname_list = set_classnames




class ClassificationMultiLabelEvaluator(EvaluatorBase):
    def compute_wmAP(self, df, am):
        """ Compute mAP weighted by the actual number ground truth class """
        class_occurrence = self._data_gt.occurrence(lambda rec: am.get_classname(rec))
        class_weights = {key:val / sum(class_occurrence.values()) for key, val in class_occurrence.items()}
        wmAP = sum([class_weights[row.name] * row.AP for _, row in df.iterrows()])
        return wmAP

    def get_am(self):
        if not hasattr(self, 'am'):
            self.am = AnnotationManager(self._data_gt.class_dict)
            classnames = self.am._all_classes if not self._classname_list else self._classname_list
            self.am.setup_distribution_type(distribution_type='multilabel', distribution_classes=classnames)
        return self.am

    def get_distribution_list(self):
        self.get_am()
        self._check_data()

        # collect data for ap computation
        annotation_distribution_list = list()
        prediction_distribution_list = list()
        for rec_anno, rec_pred in self._data_gt.zip(self._data_pred):
            dist_anno = self.am.get_distribution(rec_anno)
            dist_pred = self.am.get_distribution(rec_pred)
            annotation_distribution_list.append(dist_anno)
            prediction_distribution_list.append(dist_pred)
            
        return annotation_distribution_list, prediction_distribution_list

    def eval_miss_fa(self, threshold, class_list=None):
        annotation_distribution_list, prediction_distribution_list = self.get_distribution_list()
        classnames = self.am.classnames
        if class_list is None:
            class_list = self.am.classnames

        for class_name in class_list:
            assert class_name in classnames, f"classname {class_name} not in classnames {classnames}."
        index_list = self.am.get_classid(class_list)

        interested_ann_array = np.array(annotation_distribution_list)[:, index_list]
        interested_pred_array = (np.array(prediction_distribution_list)[:, index_list] > threshold).astype('float64')
        
        class_results = dict()
        
        for index in range(len(class_list)):
            ann_array = interested_ann_array[:, index]
            pred_array = interested_pred_array[:, index]
            info_dict = get_TFPN(ann_array, pred_array)

            class_results[class_list[index]] = info_dict
        return class_results


    def eval_ap(self, verbose=True):
        annotation_distribution_list, prediction_distribution_list = self.get_distribution_list()

        # compute ap
        ap_dict = ClassifierEvalMultilabel.compute_ap(
            y_true  = np.array(annotation_distribution_list),
            y_score = np.array(prediction_distribution_list),
            class_name_list = self.am.classnames
        )
        inflection_info = ClassifierEvalMultilabel.get_inflection_info(
            y_true  = np.array(annotation_distribution_list),
            y_score = np.array(prediction_distribution_list),
            class_name_list = self.am.classnames
        )
        p_at_r_99 = ClassifierEvalMultilabel.compute_p_at_r(            
            y_true  = np.array(annotation_distribution_list),
            y_score = np.array(prediction_distribution_list),
            class_name_list = self.am.classnames,
            recall_thresh=0.99
        )
        thr_at_p_99 = ClassifierEvalMultilabel.get_threshold_at_p(            
            y_true  = np.array(annotation_distribution_list),
            y_score = np.array(prediction_distribution_list),
            class_name_list = self.am.classnames,
            precision_thresh=0.99
        )

        # print evaluation
        if verbose:
            df = pd.DataFrame.from_dict(ap_dict, orient='index', columns=['AP'])
            df_notnull = df[df['AP'].notnull()]  # remove NaN data (No samples of this class in gt)
            print(df_notnull)

            if len(df_notnull) > 1:
                # ap info
                mAP = df_notnull.mean()['AP']
                wmAP = self.compute_wmAP(df_notnull, self.am)
                print('---------------')
                print('AP info')
                print(' mAP: {:.3f}'.format(mAP))
                print('wmAP: {:.3f}\n'.format(wmAP))

                # inflection info
                print()
                print('---------------')
                print('inflection info')
                df_inflection = pd.DataFrame.from_dict(inflection_info).T
                print(df_inflection)
                
                # p_at_r_99
                print()
                print('---------------')
                print('precision at recall 0.99')
                p_at_r_99_pd = pd.DataFrame.from_dict(p_at_r_99, orient='index', columns=['precision_at_recall_0.99'])
                print(p_at_r_99_pd)
                
                # threshold at precision 0.99
                print()
                print('---------------')
                print('threshold at precision 0.99')
                thr_at_p_99_pd = pd.DataFrame.from_dict(thr_at_p_99, orient='index', columns=['threshold_at_precision_0.99'])
                print(thr_at_p_99_pd)

        return ap_dict

    eval = eval_ap
    evaluate = eval_ap
    evaluation = eval_ap


ClassificationEvaluator = ClassificationMultiLabelEvaluator
MultiLabelEvaluator = ClassificationMultiLabelEvaluator


class ClassificationMultiClassEvaluator(EvaluatorBase):
    def eval(self, verbose=True):
        self._check_data()

        am = AnnotationManager(self._data_gt.class_dict)
        classnames = am._leaf_classes if not self._classname_list else self._classname_list
        am.setup_distribution_type(distribution_type='multiclass', distribution_classes=classnames)
        classnames = am.classnames

        # collect data for ap computation
        y_true = list()
        y_pred = list()
        for rec_anno, rec_pred in self._data_gt.zip(self._data_pred):
            classname_anno = am.get_classname(rec_anno)[0]
            classname_pred = am.get_classname(rec_pred)[0]
            y_true.append(classname_anno)
            y_pred.append(classname_pred)

        confusion_mat = compute_confusion_matrix(y_true=y_true, y_pred=y_pred, labels=classnames)
        accuracy = np.mean(np.array(y_true) == np.array(y_pred))

        # compute ap
        ap_dict = {
            'confusion_matrix': confusion_mat,
            'accuracy': accuracy
        }

        # print evaluation
        if verbose:
            df = pd.DataFrame(confusion_mat, index=classnames, columns=classnames)
            df = df.replace(0,'-')
            print()
            print('x-axis: prediction')
            print('y-axis: annotation')
            print(df)
            print("\nAccuracy: {:3f}\n".format(100*accuracy))


        return ap_dict

    evaluate = eval
    evaluation = eval


MultiClassEvaluator = ClassificationMultiClassEvaluator



from .utils import eval_map
class DetectionEvaluator(EvaluatorBase):
    def eval_ap(self, iou_threshold = .1, verbose=True):
        """
        :iou_threshold: the minimal iou that a TP prediction is required when overlapped with a gt bbox
        """
        self._check_data()

        am = AnnotationManager(self._data_gt.class_dict)
        classnames = am._leaf_classes if not self._classname_list else self._classname_list
        am.setup_distribution_type(distribution_type='detection', distribution_classes=classnames)
        classnames = am.classnames

        # collect data for ap computation
        detections = list()
        annotations = list()
        for rec_anno, rec_pred in self._data_gt.zip(self._data_pred):
            # collect detection results
            bboxes = np.reshape(am.get_xyxy(rec_pred),  [-1,4])
            scores = np.reshape(am.get_score(rec_pred), [-1,1])
            labels = np.reshape(am.get_classid(rec_pred), [-1])
            det = np.hstack([bboxes, scores])
            det_res = [det[labels==classid,:] for classid in range(len(am.classnames))]
            detections.append(det_res)

            # collect annotation results
            ann = dict(
                bboxes = np.reshape(am.get_xyxy(rec_anno), [-1,4]),
                labels = np.reshape(am.get_classid(rec_anno), [-1])
            )
            annotations.append(ann)

        # compute ap
        _, ap_results = eval_map(
            det_results=detections,
            annotations=annotations,
            iou_thr=iou_threshold,
        )

        # print evaluation
        if verbose:
            ap_dict = {classname:x['ap'] for classname, x in zip(classnames, ap_results)}
            df = pd.DataFrame.from_dict(ap_dict, orient='index', columns=['AP'])
            df_notnull = df[df['AP'].notnull()]  # remove NaN data (No samples of this class in gt)
            print(df_notnull)

        return ap_results


    eval = eval_ap
    evaluate = eval_ap
    evaluation = eval_ap



import numpy as np
import torch
from collections.abc import Sequence


def to_tensor(data):
    """Convert objects of various python types to :obj:`torch.Tensor`.

    Supported types are: :class:`numpy.ndarray`, :class:`torch.Tensor`,
    :class:`Sequence`, :class:`int` and :class:`float`.

    Args:
        data (torch.Tensor | numpy.ndarray | Sequence | int | float): Data to
            be converted.
    """

    if isinstance(data, torch.Tensor):
        return data
    elif isinstance(data, np.ndarray):
        return torch.from_numpy(data)
    elif isinstance(data, Sequence):
        return torch.tensor(data)
    elif isinstance(data, int):
        return torch.LongTensor([data])
    elif isinstance(data, float):
        return torch.FloatTensor([data])
    else:
        raise TypeError(f'type {type(data)} cannot be converted to tensor.')


def image_to_tensor(img):
    """Convert image to :obj:`torch.Tensor`.

    The dimension order of input image is (H, W, C). The pipeline will convert
    it to (C, H, W). If only 2 dimension (H, W) is given, the output would be
    (1, H, W).

    """

    if len(img.shape) < 3:
        img = np.expand_dims(img, -1)
    result = to_tensor(img.transpose(2, 0, 1))
    result = result.contiguous()
    return result



from lib.data_manager import DataManager
from lib.anno_manager import AnnotationManager
from copy import deepcopy

class_dict = [
    {'class_id': 0, 'class_name': 'NG', 'parent': None},
    {'class_id': 1, 'class_name': 'word', 'parent': None},
    {'class_id': 2, 'class_name': 'number', 'parent': 'word'},
    {'class_id': 3, 'class_name': 'character', 'parent': 'word'},
    {'class_id': 4, 'class_name': 'dirt', 'parent': 'NG'},
    {'class_id': 5, 'class_name': 'dent', 'parent': 'NG'},
    {'class_id': 6, 'class_name': 'scratch', 'parent': 'NG'},
]

## Easy annotation generation
am = AnnotationManager(class_dict)


#
# detection mode
#
assert am._distribution_classes == am._leaf_classes
assert am._distribution_type == 'detection'
am.setup_distribution_type(
    distribution_classes = ['dirt', 'dent', 'number']
)
assert am._distribution_type == 'detection'
distribution_classes = deepcopy(am._distribution_classes)

assert len(am.get_classname([])) == 0
assert set(['dent']) == set(am.get_classname([0, 1, 0]))
assert set(['number']) == set(am.get_classname([0,.2,.3]))
assert set(['dirt', 'dent']) == set(am.get_classname([.2,.2,.13]))

try:
    am.setup_distribution_type(distribution_classes = ['dirt', 'dent', 'NG'])
except Exception as e:
    print("DETECTION MODE: ", e)
else:
    raise RuntimeError("Class conflict should be detected but it is not")




#
# multiclass mode
#
am.setup_distribution_type(
    distribution_type = 'multiclass',
    distribution_classes = ['dirt', 'dent', 'word']
)

assert set(['OK']) == set(am.get_classname([]))
assert set(['dent']) == set(am.get_classname([0,1,0]))
assert set(['word']) == set(am.get_classname([.1,.1,.8]))



am.setup_distribution_type(
    distribution_classes = ['OK', 'dirt', 'dent', 'word']
)
assert set(['OK']) == set(am.get_classname([.5,0,.2,.3]))
assert set(['OK', 'word']) == set(am.get_classname([.3,.2,.2,.3]))



try:
    am.setup_distribution_type(
        distribution_classes = ['dirt', 'dent', 'NG']
    )
except Exception as e:
    print("MULTICLASS MODE: ", e)
else:
    raise RuntimeError("Class conflict should be detected but it is not")





from multiprocessing import Pool
from multiprocessing import Manager
import time
import numpy as np
from tqdm import tqdm


def single_process(main_fun, paras, share_list, share_lock, total_size, start_time, catch_exception):

    if catch_exception:
        try:
            res = main_fun(paras)
        except Exception as e:
            print(e)
            return
    else:
        res = main_fun(paras)
    if res is not None:
        share_lock.acquire()
        share_list.append(res)
        share_lock.release()
    else:
        print("None res got, and it won't appear in the share_list. ")

    print(f"{len(share_list)} / {total_size}")
    
    min_time = (time.time() - start_time) / max(1, len(share_list)) * (total_size - max(1, len(share_list))) / 60
    print(f"time left {min_time} min")


def multi_process(main_fun, para_fun, total_size, max_pool_num=8, catch_exception=True):
    start_time = time.time()
    if max_pool_num > 0:
        mp_manager = Manager()
        share_list = mp_manager.list()
        share_lock = mp_manager.Lock()
        pool = Pool(max_pool_num)
        for index in range(total_size):
            paras = para_fun(index)
            pool.apply_async(func=single_process, args=(main_fun, paras, share_list, share_lock, total_size, start_time, catch_exception))
        pool.close()
        pool.join()
        result_list = list(share_list)
    else:
        result_list = list()
        for index in range(total_size):
            res = main_fun(para_fun(index))
            result_list.append(res)
    print(f"mean time {(time.time() - start_time) / max(1, len(result_list))}")
    return result_list


if __name__ == '__main__':
    def para_gen(index):
        para_1 = np.random.randint(0, 200)
        para_2 = np.random.randint(0, 200)
        return {'para_1':para_1, 'para_2':para_2}

    def main_fun_v(para_dict):
        para_1 = para_dict['para_1']
        para_2 = para_dict['para_2']
        return para_1 + para_2

    res_list = multi_process(main_fun_v, para_gen, 200, 0)
    pass


from pathlib import Path

from .labelme import Labelme
from .labelme import Polygon
from ...utils_vvd import create_uuid
from ...utils_vvd import cv_rgb_imread


def make_ok_labelme(image_path, load_exist=False):
    if Path(image_path).exists():
        json_path = Path(image_path).with_suffix('.json')
        if not json_path.exists() or not load_exist:
            uuid = create_uuid()
            realative_image_path = Path(image_path).name
            image = cv_rgb_imread(image_path)
            H, W = image.shape[:2]

            info = {
                'uuid': uuid,
                'image_path': realative_image_path,
                'height': H, 'width': W,
                'roi': [0, 0, H, W],  # roi in parent_image: xyxy
                'parent_uuid': uuid,
            }
            labelme_info = Labelme(info, [])
        else:
            labelme_info = Labelme.from_json(json_path)
        return labelme_info
    return None


def add_shapes_to_labelme(labelme_obj, shape_list, classname_list, shape_type_list=None):
    assert isinstance(labelme_obj, Labelme), f"labelme_obj {labelme_obj} must be instance of Labelme."
    assert len(shape_list) == len(classname_list), f"boxes number {len(shape_list)} != classname number {len(classname_list)}"
    if shape_type_list is not None:
        assert len(shape_type_list) == len(classname_list), f"shape_type number {len(shape_type_list)} != classname number {len(classname_list)}"
    else:
        shape_type_list = ['polygon'] * len(classname_list)
    for shape, classname, shape_type in zip(shape_list, classname_list, shape_type_list):
        polygon_obj = Polygon(shape, classname, shape_type)
        labelme_obj.shape_list.append(polygon_obj)



from eval_metrics import ClassifierEvalBinary, ClassifierEvalMulticlass, ClassifierEvalMultilabel
from utils import label_to_onehot
import pickle as pkl


if __name__ == '__main__':

    # . Eval binary classification
    print('Evaluate binary classification...')
    label_file = './input/binary/binary_test_label.pkl'
    pred_file = './input/binary/binary_test_ep14_prediction.pkl'
    path_file = './input/binary/binary_test_path.pkl'
    with open(label_file, 'rb') as f:  # list
        labels = pkl.load(f)
    labels = [lab[0] for lab in labels]  # [[0], [1], [0]] -> [0, 1, 0]
    with open(pred_file, 'rb') as f:  # numpy
        predicts = pkl.load(f)
    with open(path_file, 'rb') as f:
        paths = pkl.load(f)
    ap_dict = ClassifierEvalBinary.compute_ap(labels, predicts)
    prec_dict = ClassifierEvalBinary.compute_p_at_r(
        labels, predicts, recall_thresh=0.995)
    ClassifierEvalBinary.draw_pr_curve(labels, predicts, output_path='./pr_curve/binary/pr.png')
    min_score, res_dict, fn_index_list, fp_index_list = ClassifierEvalBinary.compute_fnr_and_fpr(
        labels, predicts, fnr_thresh=0.005, fail_study=True)
    print('AP res is: ', ap_dict)
    print('P at R res is: ', prec_dict)
    print('Loubao and wubao res is: ', res_dict)
    print('Min ng score is: ', min_score)
    cls_dict = {0:'OK', 1:'NG'}
    ClassifierEvalBinary.draw_failure_cases(
        paths, labels, predicts, min_score, fn_index_list, fp_index_list, cls_dict, res_dir='./failure_case/binary')


    # . Eval multiclass classification
    print('Evaluate multiclass classification...')
    label_file = './input/multiclass/multiclass_test_label.pkl'
    pred_file = './input/multiclass/ep12_test_multiclass_prediction.pkl'
    path_file = './input/multiclass/multiclass_test_path.pkl'
    with open(label_file, 'rb') as f:  # list
        labels = pkl.load(f)
    labels = [lab[0] for lab in labels]  # [[0], [1], [2]] -> [0, 1, 2]
    with open(pred_file, 'rb') as f:  # numpy
        predicts = pkl.load(f)
    with open(path_file, 'rb') as f:
        paths = pkl.load(f)
    ap_dict = ClassifierEvalMulticlass.compute_ap(labels, predicts)
    prec_dict = ClassifierEvalMulticlass.compute_p_at_r(
        labels, predicts, recall_thresh=0.995)
    ClassifierEvalMulticlass.draw_pr_curve(labels, predicts, output_dir='./pr_curve/multiclass')
    min_score, res_dict, fn_index_list, fp_index_list = ClassifierEvalMulticlass.compute_fnr_and_fpr(
        labels, predicts, ok_ind=0, fnr_thresh=0.005, fail_study=True)
    print('AP res is: ', ap_dict)
    print('P at R res is: ', prec_dict)
    print('Loubao and wubao res is: ', res_dict)
    print('Min ng score is: ', min_score)
    cls_dict = {0:'ok', 1:'0', 2:'1', 3:'2', 4:'3', 5:'8'}
    ClassifierEvalMulticlass.draw_failure_cases(
        0, paths, labels, predicts, min_score, fn_index_list, fp_index_list, cls_dict, res_dir='./failure_case/multiclass')


    # . Eval multilabel classification
    print('Evaluate multilabel classification...')
    label_file = './input/multilabel/multilabel_test_label.pkl'
    pred_file = './input/multilabel/multilabel_test_ep15_prediction.pkl'
    path_file = './input/multilabel/multilabel_test_path.pkl'
    with open(label_file, 'rb') as f:  # list
        labels = pkl.load(f)
    # [[0], [1, 2], [0]] -> [[1, 0, 0], [0, 1, 1], [1, 0, 0]]
    labels = label_to_onehot(labels, 6)
    with open(pred_file, 'rb') as f:  # numpy
        predicts = pkl.load(f)
    with open(path_file, 'rb') as f:
        paths = pkl.load(f)
    ap_dict = ClassifierEvalMultilabel.compute_ap(labels, predicts)
    prec_dict = ClassifierEvalMultilabel.compute_p_at_r(
        labels, predicts, recall_thresh=0.995)
    ClassifierEvalMultilabel.draw_pr_curve(labels, predicts, output_dir='./pr_curve/multilabel')
    min_score, res_dict, fn_index_list, fp_index_list = ClassifierEvalMultilabel.compute_fnr_and_fpr(
        labels, predicts, ok_ind=0, fnr_thresh=0.005, fail_study=True)
    print('AP res is: ', ap_dict)
    print('P at R res is: ', prec_dict)
    print('Loubao and wubao res is: ', res_dict)
    print('Min ng score is: ', min_score)
    cls_dict = {0:'ok', 1:'0', 2:'1', 3:'2', 4:'3', 5:'8'}
    ClassifierEvalMultilabel.draw_failure_cases(
        0, paths, labels, predicts, min_score, fn_index_list, fp_index_list, cls_dict, res_dir='./failure_case/multilabel')
